-- Run Wireshark from Wireshark_folder, it should load the plugin automatically

-- Author		: 	Jorge Sanchez, v1.0, 01.18th.2015, BR SPY project - ETH wrapper
-- Reviwed      :   Affan  Ali, v.2.0, 04.5th.2016, BR SPY project - ETH wrapper with updated Time Stamp Field
-- Revision		:   Gerardo Medina, v3.0, 09.12th.2016, BR SPY project - ETH wrapper for BR detecting Ethernet frame in payload
-- Revision 	:   Gerardo Medina, v4.0, 07.8th.2018, BRSPY project - timestamp shown in nanoseconds 

do
    local br_spy = Proto("100Base-T1_SPY-100Base-T1", "100Base-T1_SPY, 100Base-T1")
    local eth_v1 = br_spy.fields
	eth_v1.direction = ProtoField.uint64("br_spy.direction", "direction", base_HEX)
    eth_v1.time_stamp = ProtoField.uint64("br_spy.time_stamp", "TimeStamp (ns)", base_HEX)
    eth_v1.rx_port = ProtoField.uint8("br_spy.rx_port", "Reception Port", base_HEX)
    eth_v1.extra =  ProtoField.uint8("br_spy.extra", "Extra", base_HEX)
    eth_v1.length =  ProtoField.uint16("br_spy.length", "Length (bytes)", base_HEX)
	eth_v1.payload = ProtoField.bytes("br_spy.payload", "Payload")
	eth_v1.crc = ProtoField.bytes("br_spy.crc", "CRC")	
    
	function br_spy.dissector(buf, pkt, root)
  --modify wireshark columns
        pkt.cols.protocol = br_spy.name
        local pktLength = buf:len()
        local tree = root:add(br_spy, buf(0))
		local tree_direction = tree:add(eth_v1.direction, buf(0, 1))
		local direction = buf(0,1):uint()
		if direction == 1 then	
			tree_direction:append_text(", RX  ")
			local time_stamp_value = buf(1, 6):uint64()
			time_stamp_value = time_stamp_value * 40
			tree:add(eth_v1.time_stamp, time_stamp_value)
			local tree_rx_port = tree:add(eth_v1.rx_port, buf(7, 1))
			local rx_port = buf(7, 1):uint()
			if rx_port == 1 then
				tree_rx_port:append_text(", reception port: BR_1A ")
			elseif rx_port == 2 then
				tree_rx_port:append_text(", reception port: BR_1B ")
			elseif rx_port == 3 then
				tree_rx_port:append_text(", reception port: BR_2A ")
			elseif rx_port == 4 then
				tree_rx_port:append_text(", reception port: BR_2B ")
			elseif rx_port == 5 then
				tree_rx_port:append_text(", reception port: BR_3A ")
			elseif rx_port == 6 then
				tree_rx_port:append_text(", reception port: BR_3B ")
			elseif rx_port == 7 then
				tree_rx_port:append_text(", reception port: BR_4A ")
			elseif rx_port == 8 then
				tree_rx_port:append_text(", reception port: BR_4B ")
			elseif rx_port == 9 then
				tree_rx_port:append_text(", reception port: BR_5A ")
			elseif rx_port == 10 then
				tree_rx_port:append_text(", reception port: BR_5B ")
			elseif rx_port == 11 then
				tree_rx_port:append_text(", reception port: BR_6A ")
			elseif rx_port == 12 then
				tree_rx_port:append_text(", reception port: BR_6B ")
			else
				tree_rx_port:append_text(", not recognized value!")
			end

			tree:add(eth_v1.extra, buf(8, 1)) 
		
			tree:add(eth_v1.length, buf(9, 2)) 
			
			--BR
			if rx_port == 1 or  rx_port == 2 or rx_port == 3 or rx_port == 4 or rx_port == 5 or
				rx_port == 6 or  rx_port == 7 or rx_port == 8 or rx_port == 9 or rx_port == 10 or
				rx_port == 11 or rx_port == 12 then
				local payload_tree = tree:add(eth_v1.payload, buf(11, buf:len() - 15))
				Dissector.get("eth_withfcs"):call(buf:range(11):tvb(),pkt,payload_tree)
				local crc_tree = tree:add(eth_v1.crc, buf(buf:len() - 4, 4))

			else
				tree:append_text(", not recognized value!")
			end

		else
			tree_direction:append_text(", TX  ")
		end
	end

	local eth_table = DissectorTable.get("ethertype")
	eth_table:add(8322, br_spy)
	
end