-- Run Wireshark from Wireshark_folder, it should load the plugin automatically

-- Author		: 	Jorge Sanchez, v1.0, 01.18th.2015, BR SPY project - ETH wrapper
-- Reviwed      :   Affan  Ali, v.2.0, 04.5th.2016, BR SPY project - ETH wrapper with updated Time Stamp Field
-- Revision		:   Gerardo Medina, v3.0, 16.8th.2016, BR SPY project - ETH wrapper for CAN
-- Revision		:   Gerardo Medina, v4.0, 30.11.2016, BR SPY project - ETH wrapper for CAN TX
-- Revision		:	Gerardo Medina, v5.0, 03.08.2018, BR SPY project - new BRSPY CAN header, and timestamp shown in nanoseconds 
-- Revision		:	Gerardo Medina, v6.0, 29.05.2019, BR SPY project - new BRSPY CAN TX header, and timestamp shown in nanoseconds 


do

    local can_spy = Proto("100Base-T1_SPY-CAN", "100Base-T1_SPY, CAN")
    local eth_v1 = can_spy.fields
	eth_v1.direction = ProtoField.bytes("can_spy.direction", "Direction", base_HEX)
    eth_v1.time_stamp = ProtoField.uint64("can_spy.time_stamp", "TimeStamp (ns)", base_HEX)
    eth_v1.can_bus = ProtoField.uint8("can_spy.can_bus", "Port", base_HEX)
	eth_v1.rx_port = ProtoField.uint8("can_spy.rx_port", "Reception Port", base_HEX)
	eth_v1.tx_port = ProtoField.uint8("can_spy.tx_port", "Transmission Port", base_HEX)
    eth_v1.extra =  ProtoField.uint8("can_spy.extra", "Extra", base_HEX)
	eth_v1.msg_id =  ProtoField.uint8("can_spy.msg_id", "Message ID", base_HEX)
    eth_v1.length =  ProtoField.uint16("can_spy.length", "Message Length (bytes)", base_HEX)
	eth_v1.payload = ProtoField.bytes("can_spy.payload", "Payload")
	eth_v1.crc = ProtoField.uint32("can_spy.crc", "CRC", base_HEX)	
	
	
	--CANRX
	eth_v1.canstate = ProtoField.bytes("can_spy.canstate", "CAN RX State", base_HEX)
	eth_v1.canstate_ack = ProtoField.uint8("can_spy.canstate_ack", "ACK", base_DEC)
	eth_v1.canstate_crc = ProtoField.uint8("can_spy.canstate_crc", "CRC ERROR", base_DEC)
	eth_v1.canstate_rtr = ProtoField.uint8("can_spy.canstate_rtr", "RTR", base_DEC)
	eth_v1.canstate_ide = ProtoField.uint8("can_spy.canstate_ide", "IDE", base_DEC)
	eth_v1.canstate_edl = ProtoField.uint8("can_spy.canstate_edl", "EDL", base_DEC)
	eth_v1.canstate_brs = ProtoField.uint8("can_spy.canstate_BRS", "BRS", base_DEC)
	eth_v1.canstate_esi = ProtoField.uint8("can_spy.canstate_esi", "ESI", base_DEC)

	--CANTX															
	eth_v1.canid = ProtoField.uint16("can_spy.canid", "CAN ID", base_DEC)	
	eth_v1.canlength = ProtoField.uint8("can_spy.canlength", "CAN Frame Length", base_DEC)	
	eth_v1.canpayload = ProtoField.bytes("can_spy.canpayload", "CAN Frame Payload", base_HEX)
	eth_v1.cancrc = ProtoField.bytes("can_spy.cancrc", "CAN CRC", base_HEX)
	eth_v1.canevent = ProtoField.bytes("can_spy.canevent", "CAN TX STATE", base_HEX)

    
	function can_spy.dissector(buf, pkt, root)
  --modify wireshark columns
  
        pkt.cols.protocol = can_spy.name
        local pktLength = buf:len()
        local tree = root:add(can_spy, buf(0))
		local tree_direction = tree:add(eth_v1.direction, buf(0, 1))
		local direction = buf(0,1):uint()
		
		--CANRX
		if direction == 1 then	
			tree_direction:append_text(", RX  ")
			local time_stamp_value = buf(1, 6):uint64()
			time_stamp_value = time_stamp_value * 40
			tree:add(eth_v1.time_stamp, time_stamp_value)
			local tree_rx_port = tree:add(eth_v1.rx_port, buf(7, 1))
			local rx_port = buf(7, 1):uint()

			if rx_port == 1 then
				tree_rx_port:append_text(", CAN_A  ")
			elseif rx_port == 2 then
				tree_rx_port:append_text(", CAN_B ")
			elseif rx_port == 3 then
				tree_rx_port:append_text(", CAN_C ")
			elseif rx_port == 4 then
				tree_rx_port:append_text(", CAN_D ")
			elseif rx_port == 5 then
				tree_rx_port:append_text(", CAN_E ")			
			else
				tree_rx_port:append_text(", not recognized value!")
			end

			tree:add(eth_v1.extra, buf(8, 1)) 
		
			tree:add(eth_v1.length, buf(9, 2)) 
					
			local remote = 0
			local tree_can_frames = tree:add(eth_v1.payload, buf(11, buf:len() - 11))
			local tree_can_state = tree_can_frames:add(eth_v1.canstate, buf(11,1))
			tree_can_state : add(eth_v1.canstate_ack,buf(11,1):bitfield(7,1))
			tree_can_state : add(eth_v1.canstate_crc,buf(11,1):bitfield(6,1))
			tree_can_state : add(eth_v1.canstate_rtr,buf(11,1):bitfield(5,1))
			remote = buf(11,1):bitfield(5,1)
			tree_can_state : add(eth_v1.canstate_ide,buf(11,1):bitfield(4,1))
			tree_can_state : add(eth_v1.canstate_edl,buf(11,1):bitfield(3,1))
			tree_can_state : add(eth_v1.canstate_brs,buf(11,1):bitfield(2,1))
			tree_can_state : add(eth_v1.canstate_esi,buf(11,1):bitfield(1,1))
			tree_can_frames:add(eth_v1.canid, buf(12,4))
			tree_can_frames:add(eth_v1.canlength, buf(16, 1))
			local can_length = 0	
			can_length = buf(16,1):uint()
			if remote == 0 and can_length > 0 then
				tree_can_frames:add(eth_v1.canpayload, buf(17,can_length))
				tree_can_frames:add(eth_v1.cancrc, buf(17 + can_length,3))
			else	
				tree_can_frames:add(eth_v1.cancrc, buf(17,3))
			end 
			
			
		--CANTX DATA FRAME
		elseif direction == 0 then
			tree_direction:append_text(", TX: DATA FRAME  ")
			
			local tree_tx_port = tree:add(eth_v1.tx_port, buf(1, 1))
			local can_bus = buf(1, 1):uint()
			if can_bus == 1 then
				tree_tx_port :append_text(", CAN_A ")
			elseif can_bus == 2 then
				tree_tx_port :append_text(", CAN_B ")
			elseif can_bus == 3 then
				tree_tx_port :append_text(", CAN_C ")
			elseif can_bus == 4 then
				tree_tx_port :append_text(", CAN_D ")
			elseif can_bus == 5 then
				tree_tx_port :append_text(", CAN_E ")			
			else
				tree_tx_port :append_text(", not recognized value!")
			end
			tree:add(eth_v1.msg_id, buf(2, 1)) 
			tree:add(eth_v1.length, buf(3, 2)) 
			local tree_can_frames = tree:add(eth_v1.payload, buf(5, buf:len() - 5))
			tree_can_frames:add(eth_v1.canid, buf(5,4))	
			tree_can_frames:add(eth_v1.canlength, buf(9, 1))
			local can_length = 0	
			can_length = buf(9,1):uint()
			if can_length > 0 then
				tree_can_frames:add(eth_v1.canpayload, buf(10,can_length))
			end 
			

		--CANTX EVENT FRAME	
		elseif direction == 17 then	
			tree_direction:append_text(", TX: EVENT FRAME  ")
			local time_stamp_value = buf(1, 6):uint64()
			time_stamp_value = time_stamp_value * 40
			tree:add(eth_v1.time_stamp, time_stamp_value)
			local tree_tx_port = tree:add(eth_v1.tx_port, buf(7, 1))
			local can_bus = buf(7, 1):uint()
			if can_bus == 1 then
				tree_tx_port:append_text(", CAN_A ")
			elseif can_bus == 2 then
				tree_tx_port:append_text(", CAN_B ")
			elseif can_bus == 3 then
				tree_tx_port:append_text(", CAN_C ")
			elseif can_bus == 4 then
				tree_tx_port:append_text(", CAN_D ")
			elseif can_bus == 5 then
				tree_tx_port:append_text(", CAN_E ")			
			else
				tree_tx_port:append_text(", not recognized value!")
			end			
			tree:add(eth_v1.msg_id, buf(8, 1)) 	
			tree:add(eth_v1.length, buf(9, 2)) 			
			local tree_can_frames = tree:add(eth_v1.payload, buf(11, buf:len() - 11))
			local tree_can_event = tree_can_frames:add(eth_v1.canevent, buf(11,1))
			tree_can_event : add(eth_v1.canstate_ack,buf(11,1):bitfield(7,1))
			tree_can_event : add(eth_v1.canstate_crc,buf(11,1):bitfield(6,1))
			tree_can_event : add(eth_v1.canstate_rtr,buf(11,1):bitfield(5,1))
			tree_can_event : add(eth_v1.canstate_ide,buf(11,1):bitfield(4,1))
			tree_can_event : add(eth_v1.canstate_edl,buf(11,1):bitfield(3,1))
			tree_can_event : add(eth_v1.canstate_brs,buf(11,1):bitfield(2,1))
			tree_can_event : add(eth_v1.canstate_esi,buf(11,1):bitfield(1,1))			
			tree_can_frames:add(eth_v1.canid, buf(12,4))
			tree_can_frames:add(eth_v1.canlength, buf(16, 1))
			can_length = buf(16,1):uint()
			if can_length > 0 then
				tree_can_frames:add(eth_v1.canpayload, buf(17,can_length))
				tree_can_frames:add(eth_v1.cancrc, buf(17 + can_length,3))
			else	
				tree_can_frames:add(eth_v1.cancrc, buf(17,3))
			end 
		end		
		
		
	end

	local eth_table = DissectorTable.get("ethertype")
	eth_table:add(8325, can_spy)
	
end
