-- Run Wireshark from Wireshark_folder, it should load the plugin automatically

-- Author		: 	Jorge Sanchez, v1.0, 01.18th.2015, BR SPY project - ETH wrapper
-- Reviwed      :   Affan  Ali, v.2.0, 04.5th.2016, BR SPY project - ETH wrapper with updated Time Stamp Field
-- Revision		:   Gerardo Medina, v3.0, 16.8th.2016, BR SPY project - ETH wrapper for FlexRay
-- Revision     :	Gerardo Medina, v4.0 29.5th.2018, FlexRay improved, making difference between FlexRay and BRSPY directions fields
-- Revision     :	Gerardo Medina, v5.0 07.8th.2018, timestamp shown in nanoseconds 

do

    local fr_spy = Proto("100BASE-T1_SPY-FlexRay", "100BASE-T1_SPY, FlexRay")
    local eth_v1 = fr_spy.fields
	eth_v1.direction = ProtoField.uint64("fr_spy.direction", "Direction", base_HEX)
    eth_v1.time_stamp = ProtoField.uint64("fr_spy.time_stamp", "TimeStamp (ns)", base_HEX)
    eth_v1.rx_port = ProtoField.uint8("fr_spy.rx_port", "Port", base_HEX)
    eth_v1.extra =  ProtoField.uint8("fr_spy.extra", "Extra", base_HEX)
    eth_v1.length =  ProtoField.uint16("fr_spy.length", "Length (bytes)", base_HEX)
	eth_v1.payload = ProtoField.bytes("fr_spy.payload", "Payload")
	eth_v1.crc = ProtoField.uint32("fr_spy.crc", "CRC", base_HEX)	
	
	--FlexRay

	--@@@ Header @@@
	eth_v1.cycle = ProtoField.uint16("fr_spy.cycle", "Cycle", base_DEC)
	eth_v1.msg_word_length = ProtoField.uint16("fr_spy.msg_word_length", "Message Word Length", base_DEC)	
	
	
	eth_v1.frames = ProtoField.bytes("fr_spy.frames", "FlexRay Frames", base_HEX)	
	eth_v1.padding = ProtoField.bytes("fr_spy.padding", "Padding", base_HEX)	
	
	--@@@ Frames format @@@ 
	eth_v1.frame = ProtoField.uint16("fr_spy.frame", "Frame", base_DEC)	
	eth_v1.frame_id = ProtoField.uint16("fr_spy.msg_word_length", "Frame ID", base_DEC)	
	eth_v1.fr_direction = ProtoField.uint8("fr_spy.fr_direction", "Frame Direction", base_DEC)	
	eth_v1.payload_length = ProtoField.uint8("fr_spy.msg_word_length", "Frame Word Length", base_DEC)	
	eth_v1.fr_payload = ProtoField.bytes("fr_spy.payload", "Frame Payload", base_HEX)
    
	function fr_spy.dissector(buf, pkt, root)
  --modify wireshark columns
        pkt.cols.protocol = fr_spy.name
        
        local pktLength = buf:len()
        local tree = root:add(fr_spy, buf(0))
		local tree_direction = tree:add(eth_v1.direction, buf(0, 1))
		local direction = buf(0,1):uint()
		if direction == 1 then	
			tree_direction:append_text(", RX  ")
			local time_stamp_value = buf(1, 6):uint64()
			time_stamp_value = time_stamp_value * 40
			tree:add(eth_v1.time_stamp, time_stamp_value)
			local tree_rx_port = tree:add(eth_v1.rx_port, buf(7, 1))
			local rx_port = buf(7, 1):uint()

			if rx_port == 1 then
				tree_rx_port:append_text(", reception port: FlexRay ")
				
			else
				tree_rx_port:append_text(", not recognized value!")
			end

			tree:add(eth_v1.extra, buf(8, 1)) 
		
			tree:add(eth_v1.length, buf(9, 2)) 
			
			--FlexRay
			if rx_port == 1 then
				local payload_tree = tree:add(eth_v1.payload, buf(11, buf:len() - 11))	
				--### CYCLE ###
				payload_tree:add(eth_v1.cycle, buf(11, 2))
				local cycle = buf(11,2):uint()
				--debugging info
				if cycle > 63 then
					payload_tree:append_text(", Error: invalid cycle number! ")
				end
				
				--### Message Word Length ###
				payload_tree:add(eth_v1.msg_word_length, buf(13, 2))
				local msg_word_length = buf(13,2):uint()		
				local msg_byte_length = msg_word_length * 2
				local valid_package_length = 4 + msg_byte_length
				
				local payload_tree_frames = payload_tree:add(eth_v1.frames, buf(15, msg_byte_length))
				
				--### Frames ###
				local x = 15
				local y = 1
				local frame_word_length = 0	
				local frame_index = 1
				local frame_id = 0
				local frame_direction = 0
				
				repeat 
					subtree_frame = payload_tree_frames:add(payload_tree_frames, buf(15, msg_byte_length))
					subtree_frame:set_text(string.format("Frame Number: %d", frame_index))
								
					--@@@ Frame ID @@@
					y = 2
					subtree_frame_id = subtree_frame:add(eth_v1.frame_id, buf(x,y))
					frame_id = buf(x,y):uint()
					if frame_id == 0 then
						subtree_frame_id:append_text(", Error: invalid Frame ID!")
					elseif frame_id > 2047 then
						subtree_frame_id:append_text(", Error: invalid Frame ID!")
					end			
					subtree_frame:append_text(string.format(", ID=%d", frame_id))
					x = x + y
					
					--@@@ Frame Direction @@@
					y = 1
					subsubtree_frame_direction = subtree_frame:add(eth_v1.fr_direction, buf(x,y))
					frame_direction = buf(x,y):uint()
					if frame_direction == 0 then
						subsubtree_frame_direction:append_text(", RX")
					elseif frame_direction == 1 then
						subsubtree_frame_direction:append_text(", TX")
					else
						subsubtree_frame_direction:append_text(", Error: invalid direction!")
					end
					x = x + y
					
					--@@@ Frame Length @@@
					y = 1
					subtree_frame:add(eth_v1.payload_length, buf(x,y))
					frame_word_length = buf(x,y):uint()		
					x = x + y

					--@@@ Frame Payload @@@
					y = frame_word_length * 2
					subtree_frame:add(eth_v1.fr_payload, buf(x,y))
					x = x + y
				
					frame_index = frame_index + 1
				
				until x >= valid_package_length 
				
				-- if package_length > valid_package_length then
					-- local padding_length = package_length - valid_package_length
					-- tree:add(eth_v1.padding, buf(valid_package_length, padding_length))
				-- end
			
				
			--BR
			-- elseif rx_port == 1 or  rx_port == 2 or rx_port == 3 or rx_port == 4 or rx_port == 5 or
				-- rx_port == 6 or  rx_port == 7 or rx_port == 8 or rx_port == 9 or rx_port == 10 or
				-- rx_port == 11 or rx_port == 12 then
				-- tree:add(eth_v1.payload, buf(10, buf:len() - 10))
			end
		else
			tree_direction:append_text(", TX  ")
		end		
		
	end

	local eth_table = DissectorTable.get("ethertype")
	eth_table:add(8323, fr_spy)
	
end
