-- Run Wireshark from Wireshark_folder, it should load the plugin automatically

-- Author		: 	Jorge Sanchez, v1.0, 01.18th.2015, BR SPY project - ETH wrapper
-- Reviwed      :   Affan  Ali, v.2.0, 04.5th.2016, BR SPY project - ETH wrapper with updated Time Stamp Field
-- Revision		:   Gerardo Medina, v3.0, 16.8th.2016, BR SPY project - ETH wrapper for LIN
-- Revision		:   Gerardo Medina, v4.0, 06.08th.2018, BR SPY project - New LIN control field, and timestamp in nanoseconds

do

    local lin_spy = Proto("100Base-T1_SPY-LIN", "100Base-T1_SPY, LIN")
    local eth_v1 = lin_spy.fields
	eth_v1.direction = ProtoField.uint64("lin_spy.direction", "Direction", base_HEX)
    eth_v1.time_stamp = ProtoField.uint64("lin_spy.time_stamp", "TimeStamp (ns)", base_HEX)
    eth_v1.rx_port = ProtoField.uint8("lin_spy.rx_port", "Port", base_HEX)
    eth_v1.extra =  ProtoField.uint8("lin_spy.extra", "Extra", base_HEX)
    eth_v1.length =  ProtoField.uint16("lin_spy.length", "Length (bytes)", base_HEX)
	eth_v1.payload = ProtoField.bytes("lin_spy.payload", "Payload")
	eth_v1.crc = ProtoField.uint32("lin_spy.crc", "CRC", base_HEX)	
	
	--LIN
	eth_v1.linstate = ProtoField.bytes("lin_spy.linstate", "LIN Control Field", base_HEX)
	eth_v1.linid = ProtoField.uint16("lin_spy.linid", "LIN ID", base_DEC)	
	eth_v1.linlength = ProtoField.uint8("lin_spy.linlength", "LIN Frame Length", base_DEC)	
	eth_v1.linpayload = ProtoField.bytes("lin_spy.linpayload", "LIN Payload", base_HEX)
	eth_v1.lincrc = ProtoField.bytes("lin_spy.lincrc", "LIN Checksum", base_HEX)
	eth_v1.linstate_par = ProtoField.uint8("lin_spy.linstate_par", "PARITY ERROR", base_DEC)
	eth_v1.linstate_crc = ProtoField.uint8("lin_spy.linstate_crc", "CHECKSUM ERROR", base_DEC)
	eth_v1.linstate_col = ProtoField.uint8("lin_spy.linstate_col", "COLISSION ERROR", base_DEC)
	
	eth_v1.linframes = ProtoField.bytes("lin_spy.linframes", "LIN Frame", base_HEX)	
    
	function lin_spy.dissector(buf, pkt, root)
  --modify wireshark columns
        pkt.cols.protocol = lin_spy.name
        
        local pktLength = buf:len()
        local tree = root:add(lin_spy, buf(0))
		local tree_direction = tree:add(eth_v1.direction, buf(0, 1))
		local direction = buf(0,1):uint()
		if direction == 1 then	
			tree_direction:append_text(", RX  ")
			local time_stamp_value = buf(1, 6):uint64()
			time_stamp_value = time_stamp_value * 40
			tree:add(eth_v1.time_stamp, time_stamp_value)
			local tree_rx_port = tree:add(eth_v1.rx_port, buf(7, 1))
			local rx_port = buf(7, 1):uint()
			
			if rx_port == 1 then
				tree_rx_port:append_text(", reception port: LIN ")			
			else
				tree_rx_port:append_text(", not recognized value!")
			end

			tree:add(eth_v1.extra, buf(8, 1)) 
		
			tree:add(eth_v1.length, buf(9, 2)) 

			--LIN	
			if rx_port == 1 then
				local tree_lin_frames = tree:add(eth_v1.payload, buf(11, buf:len() - 11))
				local tree_lin_state = tree_lin_frames:add(eth_v1.linstate, buf(11,1))
				tree_lin_state : add(eth_v1.linstate_crc,buf(11,1):bitfield(7,1))
				tree_lin_state : add(eth_v1.linstate_col,buf(11,1):bitfield(6,1))
				tree_lin_state : add(eth_v1.linstate_par,buf(11,1):bitfield(5,1))
				tree_lin_frames:add(eth_v1.linid, buf(12,1))
				local tree_lin_length = tree_lin_frames:add(eth_v1.linlength, buf(13, 1))
				local lin_length = 0	
				lin_length = buf(13,1):uint()
				if lin_length == 0 then
					tree_lin_length :append_text(": No answer from slave!")
				else
					tree_lin_frames:add(eth_v1.linpayload, buf(14,lin_length))
					tree_lin_frames:add(eth_v1.lincrc, buf(14 + lin_length,1))
				end
			end
		else
			tree_direction:append_text(", TX  ")
			local tree_rx_port = tree:add(eth_v1.rx_port, buf(1, 1))
			local rx_port = buf(1, 1):uint()

			if rx_port == 1 then
				tree_rx_port:append_text(", reception port: LIN  ")		
			else
				tree_rx_port:append_text(", not recognized value!")
			end
							
			 if rx_port == 1 then
				local tree_lin_frames = tree:add(eth_v1.payload, buf(2, buf:len() - 2))
				tree_lin_frames:add(eth_v1.linid, buf(2,1))
				tree_lin_frames:add(eth_v1.linlength, buf(3, 1))
				local lin_length = 0	
				lin_length = buf(3,1):uint()
				if lin_length > 0 then
					tree_lin_frames:add(eth_v1.linpayload, buf(4, lin_length))
				end 
			end
		end		
	end

	local eth_table = DissectorTable.get("ethertype")
	eth_table:add(8324, lin_spy)
	
end
